@@ -1,13 +1,26 @@
 import re
+
+import django
 from django.conf import settings
 from django.utils.html import escape
 from django.utils.encoding import smart_str
-from urlobject import URLObject
-from urlobject.query_string import QueryString
-from django.template import StringOrigin, Lexer, Parser
+from django.template.base import Lexer, Parser
+
+try:
+    from django.template import StringOrigin as Origin
+except ImportError:
+    from django.template import Origin
+
 from django.template.defaulttags import kwarg_re
 from django.template import Template, Library, Node, TemplateSyntaxError
 
+try:
+    from django.utils import six
+except ImportError:
+    import six
+
+from urlobject import URLObject
+from urlobject.query_string import QueryString
 
 register = Library()
 
@@ -17,7 +30,7 @@ TRUE_RE = re.compile(r'^(true|on)$', flags=re.IGNORECASE)
 def convert_to_boolean(string_or_boolean):
     if isinstance(string_or_boolean, bool):
         return string_or_boolean
-    if isinstance(string_or_boolean, basestring):
+    if isinstance(string_or_boolean, six.string_types):
         return bool(TRUE_RE.match(string_or_boolean))
 
 
@@ -37,7 +50,7 @@ class SpurlURLBuilder(object):
 
         self.set_sensible_defaults()
 
-        url = unicode(self.url)
+        url = six.text_type(self.url)
 
         if self.autoescape:
             url = escape(url)
@@ -57,6 +70,10 @@ class SpurlURLBuilder(object):
         base = self.prepare_value(value)
         self.url = URLObject(base)
 
+    def handle_auth(self, value):
+        auth = self.prepare_value(value)
+        self.url = self.url.with_auth(*auth.split(':', 1))
+
     def handle_secure(self, value):
         is_secure = convert_to_boolean(value)
         scheme = 'https' if is_secure else 'http'
@@ -74,7 +91,7 @@ class SpurlURLBuilder(object):
 
     def handle_add_query(self, value):
         query_to_add = self.prepare_value(value)
-        if isinstance(query_to_add, basestring):
+        if isinstance(query_to_add, six.string_types):
             query_to_add = QueryString(query_to_add).dict
         self.url = self.url.add_query_params(**query_to_add)
 
@@ -84,7 +101,7 @@ class SpurlURLBuilder(object):
 
     def handle_set_query(self, value):
         query_to_set = self.prepare_value(value)
-        if isinstance(query_to_set, basestring):
+        if isinstance(query_to_set, six.string_types):
             query_to_set = QueryString(query_to_set).dict
         self.url = self.url.set_query_params(**query_to_set)
 
@@ -93,15 +110,25 @@ class SpurlURLBuilder(object):
         self.url = self.url.set_query_params(**url.query.dict)
 
     def handle_remove_query_param(self, value):
-        self.url = self.url.del_query_param(value)
+        query_to_remove = self.prepare_value(value)
+        if '=' in query_to_remove:
+            k, v = query_to_remove.split('=', 1)
+            self.url = self.url.del_query_param_value(k, v)
+        else:
+            self.url = self.url.del_query_param(query_to_remove)
+
+    def handle_remove_query_params_except(self, value):
+        params_to_keep = self.prepare_value(value).split(" ")
+        params_to_remove = [pair[0] for pair in self.url.query_list if pair[0] not in params_to_keep]
+        self.url = self.url.with_query(self.url.query.del_params(params_to_remove))
 
     def handle_toggle_query(self, value):
         query_to_toggle = self.prepare_value(value)
-        if isinstance(query_to_toggle, basestring):
+        if isinstance(query_to_toggle, six.string_types):
             query_to_toggle = QueryString(query_to_toggle).dict
         current_query = self.url.query.dict
-        for key, value in query_to_toggle.items():
-            if isinstance(value, basestring):
+        for key, value in list(query_to_toggle.items()):
+            if isinstance(value, six.string_types):
                 value = value.split(',')
             first, second = value
             if key in current_query and first in current_query[key]:
@@ -168,9 +195,8 @@ class SpurlURLBuilder(object):
     def prepare_value(self, value):
         """Prepare a value by unescaping embedded template tags
         and rendering through Django's template system"""
-        if isinstance(value, basestring):
-            value = self.unescape_tags(value)
-            value = self.render_template(value)
+        if isinstance(value, six.string_types):
+            value = self.render_template(self.unescape_tags(value))
         return value
 
     def unescape_tags(self, template_string):
@@ -178,16 +204,23 @@ class SpurlURLBuilder(object):
         the inner templatetags are escaped - {\% and %\}"""
         return template_string.replace('{\%', '{%').replace('%\}', '%}')
 
-    def compile_string(self, template_string, origin):
+    def compile_string(self, template_string, origin, template_debug=False):
         """Re-implementation of django.template.base.compile_string
         that takes into account the tags and filter of the parser
         that rendered the parent template"""
-        if settings.TEMPLATE_DEBUG:
-            from django.template.debug import DebugLexer, DebugParser
-            lexer_class, parser_class = DebugLexer, DebugParser
+        if template_debug is True:
+            if django.VERSION < (1, 9):
+                from django.template.debug import DebugLexer, DebugParser
+                lexer_class, parser_class = DebugLexer, DebugParser
+            else:
+                from django.template.base import DebugLexer
+                lexer_class, parser_class = DebugLexer, Parser
         else:
             lexer_class, parser_class = Lexer, Parser
-        lexer = lexer_class(template_string, origin)
+        if django.VERSION < (1, 9):
+            lexer = lexer_class(template_string, origin)
+        else:
+            lexer = lexer_class(template_string)
         parser = parser_class(lexer.tokenize())
 
         # Attach the tags and filters from the parent parser
@@ -203,12 +236,16 @@ class SpurlURLBuilder(object):
         self.context.autoescape = False
 
         template = Template('')
-        if settings.TEMPLATE_DEBUG:
-            origin = StringOrigin(template_string)
+        template_debug = getattr(
+            settings, 'TEMPLATE_DEBUG', template.engine.debug if hasattr(
+                template, 'engine') else False)
+        if template_debug is True:
+            origin = Origin(template_string)
         else:
             origin = None
 
-        template.nodelist = self.compile_string(template_string, origin)
+        template.nodelist = self.compile_string(
+            template_string, origin, template_debug)
 
         rendered = template.render(self.context)
         self.context.autoescape = original_autoescape
